unit FrameRateEmulation_;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls, Menus, VidGrab;

type
  TForm1 = class(TForm)
    Image1: TImage;
    GroupBox2: TGroupBox;
    GroupBox3: TGroupBox;
    cmbVideoDevices: TComboBox;
    cmbAnalogVideoStandards: TComboBox;
    cmbVideoInputs: TComboBox;
    cmbVideoSizes: TComboBox;
    btnStart: TButton;
    btnStop: TButton;
    VideoGrabber1: TVideoGrabber;
    VideoGrabber2: TVideoGrabber;
    Label3: TLabel;
    Label5: TLabel;
    Memo1: TMemo;
    btnPause: TButton;
    Label1: TLabel;
    btnResume: TButton;
    Label2: TLabel;
    Label4: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    edtFrameRate: TEdit;
    btnFrameRate: TButton;
    GroupBox1: TGroupBox;
    btnCaptureToImage: TButton;
    btnCaptureToJPEG: TButton;
    btnCaptureToBMP: TButton;
    chkTextOverlay: TCheckBox;
    chkFreeHandEnabled: TCheckBox;
    btnEraseDrawing: TButton;
    procedure btnPauseClick(Sender: TObject);
    procedure btnResumeClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure cmbVideoDevicesChange(Sender: TObject);
    procedure cmbAnalogVideoStandardsChange(Sender: TObject);
    procedure btnStartClick(Sender: TObject);
    procedure btnCaptureToJPEGClick(Sender: TObject);
    procedure btnCaptureToBMPClick(Sender: TObject);
    procedure btnStopClick(Sender: TObject);
    procedure cmbVideoInputsChange(Sender: TObject);
    procedure cmbVideoSizesChange(Sender: TObject);
    procedure btnEraseDrawingClick(Sender: TObject);
    procedure btnCaptureToImageClick(Sender: TObject);
    procedure VideoGrabber1VideoDeviceSelected(Sender: TObject);
    procedure VideoGrabber2FrameCaptureCompleted(Sender: TObject;
      FrameBitmap: TBitmap; BitmapWidth, BitmapHeight: Integer;
      FrameNumber: Cardinal; FrameTime: Int64; DestType: TFrameCaptureDest;
      FileName: String; Success: Boolean; FrameId: Integer);
    procedure btnFrameRateClick(Sender: TObject);
    procedure chkTextOverlayClick(Sender: TObject);
    procedure VideoGrabber2FrameOverlayUsingDC(Sender: TObject; Dc: HDC;
      FrameNumber: Cardinal; FrameTime: Int64; FrameId: Integer);
    procedure VideoGrabber2MouseDown(Sender: TObject; VideoWindow: Integer;
      Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure VideoGrabber2MouseMove(Sender: TObject; VideoWindow: Integer;
      Shift: TShiftState; X, Y: Integer);
    procedure VideoGrabber2MouseUp(Sender: TObject; VideoWindow: Integer;
      Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
  private
  public
  end;

var
  Form1: TForm1;

implementation

const
   FreeHandMouseMax = 1000;

var
   FreeHandMouseDrawing: array[0..FreeHandMouseMax] of TRect;
   FreeHandMouseDrawingEnabled: boolean;
   iFreeHandMouseDrawing: LongInt;

{$R *.DFM}

procedure TForm1.FormCreate(Sender: TObject);
begin
   FreeHandMouseDrawingEnabled:= False; // used for free-hand mouse drawing
   iFreeHandMouseDrawing:= 0;           // used for free-hand mouse drawing

   cmbVideoDevices.Items.Text := VideoDevices;
   cmbVideoDevices.ItemIndex := VideoGrabber1.VideoDevice;
   Memo1.Lines.Add ('storage: ' + VideoGrabber1.StoragePath);

   // VideoGrabber1 is the master component
   VideoGrabber1.MultiplexedRole := mr_MultiplexedMaster;

   // VideoGrabber2 is the slave component
   VideoGrabber2.MultiplexedRole := mr_MultiplexedSlave;

   // this associates VideoGrabber2 to VideoGrabber1
   VideoGrabber1.AssociateMultiplexedSlave (0, VideoGrabber2.UniqueId);

   VideoGrabber2.TextOverlay_Enabled := True;
   VideoGrabber2.FrameRate := 15;
end;

procedure TForm1.btnStartClick(Sender: TObject);
begin
   VideoGrabber1.StartPreview;
end;

procedure TForm1.btnStopClick(Sender: TObject);
begin
   VideoGrabber1.StopPreview;
end;

procedure TForm1.btnPauseClick(Sender: TObject);
begin
   VideoGrabber1.PausePreview;
end;

procedure TForm1.btnResumeClick(Sender: TObject);
begin
   VideoGrabber1.ResumePreview;
end;

procedure TForm1.btnCaptureToImageClick(Sender: TObject);
begin
    VideoGrabber2.CaptureFrameTo (fc_TBitmap, '');
end;

procedure TForm1.btnCaptureToJPEGClick(Sender: TObject);
begin
   VideoGrabber2.CaptureFrameTo (fc_JpegFile, '');
end;

procedure TForm1.btnCaptureToBMPClick(Sender: TObject);
begin
   VideoGrabber2.CaptureFrameTo (fc_BmpFile, '');
end;

procedure TForm1.VideoGrabber1VideoDeviceSelected(Sender: TObject);
begin
   cmbVideoDevices.Items.Text := VideoDevices;
   cmbVideoDevices.ItemIndex := VideoGrabber1.VideoDevice;

   cmbAnalogVideoStandards.Items.Text := VideoGrabber1.AnalogVideoStandards;
   cmbAnalogVideoStandards.ItemIndex := integer (VideoGrabber1.AnalogVideoStandard);
   cmbAnalogVideoStandards.Enabled := VideoGrabber1.IsAnalogVideoDecoderAvailable;

   cmbVideoInputs.Items.Text := VideoGrabber1.VideoInputs;
   cmbVideoInputs.ItemIndex := integer (VideoGrabber1.VideoInput);

   cmbVideoSizes.Items.Text := VideoGrabber1.VideoSizes;
   cmbVideoSizes.ItemIndex := integer (VideoGrabber1.VideoSize);
end;

procedure TForm1.btnEraseDrawingClick(Sender: TObject);
begin
   iFreeHandMouseDrawing := 0;
end;

procedure TForm1.cmbVideoDevicesChange(Sender: TObject);
begin
   VideoGrabber1.VideoDevice := cmbVideoDevices.ItemIndex;
end;

procedure TForm1.cmbAnalogVideoStandardsChange(Sender: TObject);
begin
   VideoGrabber1.AnalogVideoStandard := cmbAnalogVideoStandards.ItemIndex;
end;

procedure TForm1.cmbVideoInputsChange(Sender: TObject);
begin
   VideoGrabber1.VideoInput := cmbVideoInputs.ItemIndex;
end;

procedure TForm1.cmbVideoSizesChange(Sender: TObject);
begin
   VideoGrabber1.VideoSize := cmbVideoSizes.ItemIndex;
end;

procedure TForm1.VideoGrabber2FrameCaptureCompleted(Sender: TObject;
  FrameBitmap: TBitmap; BitmapWidth, BitmapHeight: Integer;
  FrameNumber: Cardinal; FrameTime: Int64; DestType: TFrameCaptureDest;
  FileName: String; Success: Boolean; FrameId: Integer);
begin
   case DestType of
      fc_BmpFile, fc_JPEGFile: begin
        Memo1.Lines.add (FileName + ' created');
      end;
      fc_TBitmap: begin
         Image1.Picture.Bitmap.Assign (FrameBitmap);
      end;
   end;
end;

procedure TForm1.btnFrameRateClick(Sender: TObject);
begin
   VideoGrabber2.FrameRate := StrToFloatDef (edtFrameRate.Text, 15);
   VideoGrabber1.StartPreview;
end;

procedure TForm1.chkTextOverlayClick(Sender: TObject);
begin
   VideoGrabber2.TextOverlay_Enabled := chkTextOverlay.Checked;
end;

procedure TForm1.VideoGrabber2FrameOverlayUsingDC(Sender: TObject; Dc: HDC;
  FrameNumber: Cardinal; FrameTime: Int64; FrameId: Integer);
var
   Canvas: TCanvas;
   i: Integer;
begin
   if iFreeHandMouseDrawing > 0 then begin // see VideoGrabberMouseMove event below
      Canvas := TCanvas.Create;
      Canvas.Handle := Dc;
      Canvas.Pen.Color := clRed;
      for i := 0 to iFreeHandMouseDrawing - 1 do begin
         if FreeHandMouseDrawing[i].Right = 0 then begin // Right is used as boolean to know if it is a MoveTo or a LineTo
            Canvas.MoveTo (FreeHandMouseDrawing[i].Left, FreeHandMouseDrawing[i].Top);
            Canvas.Pixels[FreeHandMouseDrawing[i].Left, FreeHandMouseDrawing[i].Top] := clRed;
         end
         else begin
            Canvas.LineTo (FreeHandMouseDrawing[i].Left, FreeHandMouseDrawing[i].Top);
         end;
      end;
      Canvas.Free;
   end;
end;

procedure TForm1.VideoGrabber2MouseDown(Sender: TObject;
  VideoWindow: Integer; Button: TMouseButton; Shift: TShiftState; X,
  Y: Integer);
begin
   if VideoWindow <> -1 then begin // if the video is displayed
      if chkFreeHandEnabled.Checked then begin
         FreeHandMouseDrawing[iFreeHandMouseDrawing].Left := x;
         FreeHandMouseDrawing[iFreeHandMouseDrawing].Top := y;
         FreeHandMouseDrawing[iFreeHandMouseDrawing].Right := 0; // Right is used as boolean to know if it is a MoveTo or a LineTo
         inc (iFreeHandMouseDrawing);
         FreeHandMouseDrawingEnabled := True;
      end;
   end;
end;

procedure TForm1.VideoGrabber2MouseMove(Sender: TObject;
  VideoWindow: Integer; Shift: TShiftState; X, Y: Integer);
begin
   if VideoWindow <> -1 then begin // if the video is displayed
      if FreeHandMouseDrawingEnabled then begin
         if iFreeHandMouseDrawing < FreeHandMouseMax then begin
            FreeHandMouseDrawing[iFreeHandMouseDrawing].Left := x;
            FreeHandMouseDrawing[iFreeHandMouseDrawing].Top := y;
            FreeHandMouseDrawing[iFreeHandMouseDrawing].Right := 1; // Right is used as boolean to know if it is a MoveTo or a LineTo
            inc (iFreeHandMouseDrawing);
         end;
      end;
   end;
end;

procedure TForm1.VideoGrabber2MouseUp(Sender: TObject;
  VideoWindow: Integer; Button: TMouseButton; Shift: TShiftState; X,
  Y: Integer);
begin
   FreeHandMouseDrawingEnabled := False;
end;

end.
